SystemOrganization addCategory: #SortDescriptors!

TestCase subclass: #ClassHierarchySortDescriptorTest
	instanceVariableNames: 'objects sortedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 13:52'!
printedStringOfSortedObjects

	^String streamContents: [:stream|
		sortedObjects do: [:each| stream print: each;cr]
		]  
	! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 14:03'!
setUp

	objects := {
		Collection.
		String.
		Object.
		Array.
		}.! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 13:52'!
sortWithDescriptor: aSortDescriptor 

	sortedObjects := objects sorted: aSortDescriptor! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:47'!
testAscending

	self sortWithDescriptor: (BinarySortDescriptor classHierarchySortingDescriptor ascending: true).
	self assert: 4 equals: sortedObjects size. 
	self assert: 
'Object
Collection
String
Array
'	equals: self printedStringOfSortedObjects! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:47'!
testCollectionVsObject

	|sd|
	sd := (BinarySortDescriptor classHierarchySortingDescriptor ascending: true).
	self deny: (sd value: Collection value: Object). 
! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:47'!
testObjectVsCollection

	|sd|
	sd := (BinarySortDescriptor classHierarchySortingDescriptor ascending: true).
	self assert: (sd value: Object value: Collection).
! !

TestCase subclass: #UnarySortDescriptorTest
	instanceVariableNames: 'objects sortedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:19'!
printedStringOfSortedObjects

	^String streamContents: [:stream|
		sortedObjects do: [:each| stream print: each;cr]
		]  
	! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:11'!
setUp

	objects := {
		{100. 'Astachio'}. 
		{1. 'Betty'}. 
		{60. 'Carl'}. 
		{1. 'Duffy'}. 
		{10. 'Elizabeth'}.
		{60. 'Felix'}. 
		{5. 'Gerero'}. 
		{60. 'Howard'}. 
		{1. 'Isabell'}. 
		{6. 'Julian'}.
		}.! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:15'!
sortWithDescriptor: aSortDescriptor 

	sortedObjects := objects sorted: aSortDescriptor! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:44'!
testAscending

	self sortWithDescriptor: (UnarySortDescriptor ascendingBy: #first).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:44'!
testBlockAspect

	self sortWithDescriptor: (UnarySortDescriptor ascendingBy: [:each| each first]).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:44'!
testChangeDirection

	| sd |
	sd := UnarySortDescriptor ascendingBy: #first.
	self sortWithDescriptor: sd.
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects.
	sd changeDirection.
	self sortWithDescriptor: sd.
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(100 ''Astachio'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(10 ''Elizabeth'')
#(6 ''Julian'')
#(5 ''Gerero'')
#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
' equals: self printedStringOfSortedObjects.
	! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:44'!
testDescending

	self sortWithDescriptor: (UnarySortDescriptor descendingBy: #first).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(100 ''Astachio'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(10 ''Elizabeth'')
#(6 ''Julian'')
#(5 ''Gerero'')
#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
' equals: self printedStringOfSortedObjects! !

!UnarySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/16/2012 11:44'!
testSubDescriptor

	self sortWithDescriptor: (UnarySortDescriptor ascendingBy: #first subSortDescriptor: (UnarySortDescriptor descendingBy: #second)).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Isabell'')
#(1 ''Duffy'')
#(1 ''Betty'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Howard'')
#(60 ''Felix'')
#(60 ''Carl'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !

Object subclass: #SortDescriptor
	instanceVariableNames: 'ascending subSortDescriptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

SortDescriptor subclass: #BinarySortDescriptor
	instanceVariableNames: 'sortBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!BinarySortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:27'!
ascendingBy: aBlock

	^self new 
		sortBlock: aBlock; 
		ascending: true! !

!BinarySortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:30'!
classHierarchySortingDescriptor

	^self ascendingBy: [:class1 :class2| class2 inheritsFrom: class1].! !

!BinarySortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:28'!
descendingBy: aBlock 

	^self new 
		sortBlock: aBlock; 
		ascending: false! !

!BinarySortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 17:22'!
sortBlock: abBlock

	self assert: [ abBlock numArgs = 2].
	sortBlock := abBlock! !

!BinarySortDescriptor methodsFor: 'evaluating' stamp: 'hjo 1/16/2012 11:29'!
value: object1 value: object2

	|object2object1 object1object2|
	object2object1 := sortBlock value: object2 value: object1.
	object1object2 := sortBlock value: object1 value: object2.
	^object2object1 = object1object2
		ifTrue: [
			subSortDescriptor isNil 
				ifTrue: [ true ]
				ifFalse: [ subSortDescriptor value: object1 value: object2 ]]
		ifFalse: [ ascending xor: [object2object1] ].! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:28'!
ascendingBy: aSymbol ascendingBy: aSymbol2 ascendingBy: aSymbol3 

	^self ascendingBy: aSymbol subSortDescriptor: (self ascendingBy: aSymbol2 subSortDescriptor: (self ascendingBy: aSymbol3)).! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:28'!
ascendingBy: aSymbol subSortDescriptor: aSortDescriptor 

	^(self ascendingBy: aSymbol)
		subSortDescriptor: aSortDescriptor   ! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 1/8/2012 17:28'!
descendingBy: aSymbol subSortDescriptor: aSortDescriptor 

	^(self descendingBy: aSymbol)
		subSortDescriptor: aSortDescriptor   ! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
ascending
	^ ascending! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
ascending: aBoolean
	ascending := aBoolean! !

!SortDescriptor methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 17:46'!
chainSubSortDescriptors: anArray 

	| sd |
	sd := self.
	anArray do: [:each|
		sd subSortDescriptor: each.
		sd := each ]! !

!SortDescriptor methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:26'!
changeDirection

	ascending := ascending not! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:27'!
subSortDescriptor
	^ subSortDescriptor! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:27'!
subSortDescriptor: anObject
	subSortDescriptor := anObject! !

SortDescriptor subclass: #UnarySortDescriptor
	instanceVariableNames: 'aspect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!UnarySortDescriptor commentStamp: 'hjo 1/8/2012 15:29' prior: 0!
(NOTE) I will be renamed to AspectSortDescriptor, and superclass will take my name.

I can work as a sort-block of SortedCollection.

Smalltalk sort-blocks have two arguments and describes boolean expression that defines the sorting.
I am instanticated differently. You specify the sorting direction and aspect. An aspect is used to access value from the objects to be sorted.

For example,
anArray sorted: (SortDescriptor ascendingBy: #price).

Aspect is can be symbol of one argument or block of one argument. Above example can be rewritten with block like below.

anArray sorted: (SortDescriptor ascendingBy: [:obj| obj price]).

In addition to that, I can change ascending/descending mode. So it is very easy thing to toggle sorting direction in the list view.

sd := SortDescriptor ascendingBy: #price.
anArray sorted: sd. "ascending sort"
sd ascending: false.
anArray sorted: sd. "descending sort"


In addition to that, I support multi-sorting by using subSortDescriptor.  
anArray sorted: (SortDescriptor ascendingBy: #price subSortDescriptor: (SortDescriptor descendingBy: [:x| (x perform: #parent) name]) )!

!UnarySortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 19:54'!
ascendingBy: aSymbol 

	^self new 
		aspect: aSymbol; 
		ascending: true! !

!UnarySortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 20:36'!
descendingBy: aSymbol 

	^self new 
		aspect: aSymbol; 
		ascending: false! !

!UnarySortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
aspect
	^aspect! !

!UnarySortDescriptor methodsFor: 'accessing' stamp: 'hjo 11/3/2011 19:36'!
aspect: aSymbol
	aspect := aSymbol! !

!UnarySortDescriptor methodsFor: 'evaluating' stamp: 'hjo 11/3/2011 20:32'!
value: anObject1 value: anObject2 

	| v1 v2 |
	v1 := aspect value: anObject1.
	v2 := aspect value: anObject2.
	^v1 = v2
		ifTrue: [ subSortDescriptor isNil or: [ subSortDescriptor value: anObject1 value: anObject2 ] ]
		ifFalse: [ ascending xor: [v1 > v2] ]! !
